=== modified file 'NEWS'
--- NEWS	2005-09-01 11:27:20 +0000
+++ NEWS	2005-09-12 01:29:49 +0000
@@ -1,4 +1,33 @@
-bzr development head
+development head
+
+  IMPROVEMENTS:
+
+    * Adding a file whose parent directory is not versioned will
+      implicitly add the parent, and so on up to the root. This means
+      you should never need to explictly add a directory, they'll just
+      get added when you add a file in the directory.  Contributed by
+      Michael Ellerman.
+
+
+  INTERNALS:
+
+    * Refactor xml packing/unpacking.
+
+
+  BUG FIXES: 
+
+    * Fixed 'bzr mv' by Ollie Rutherfurd.
+
+    * Fixed strange error when trying to access a nonexistent http
+      branch.
+
+
+  PORTABILITY:
+
+    * Various Windows fixes from Ollie Rutherfurd.
+
+
+bzr-0.0.7 2005-09-02
 
   NEW FEATURES:
 
@@ -26,11 +55,18 @@
 
     * Fix case of $EDITOR containing options for the editor.
 
+    * Fix log -r refusing to show the last revision.
+      (Patch from Goffredo Baroncelli.)
+
 
   CHANGES:
 
     * ``bzr log --show-ids`` shows the revision ids of all parents.
 
+    * Externally provided commands on your $BZRPATH no longer need
+      to recognize --bzr-usage to work properly, and can just handle
+      --help themselves.
+
 
   LIBRARY:
 

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-08-25 12:46:42 +0000
+++ bzrlib/__init__.py	2005-09-05 04:07:23 +0000
@@ -38,7 +38,7 @@
 
 __copyright__ = "Copyright 2005 Canonical Development Ltd."
 __author__ = "Martin Pool <mbp@canonical.com>"
-__version__ = '0.0.7pre'
+__version__ = '0.1pre'
 
 
 def get_bzr_revision():

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-09-01 07:28:14 +0000
+++ bzrlib/add.py	2005-09-12 01:29:49 +0000
@@ -18,6 +18,7 @@
 from bzrlib.errors import NotBranchError
 from bzrlib.branch import Branch
 from bzrlib.osutils import quotefn
+from os.path import dirname
 
 def glob_expand_for_win32(file_list):
     import glob
@@ -78,11 +79,8 @@
     Returns the number of files added.
     """
     import os
-    import sys
-    from bzrlib.osutils import quotefn
     from bzrlib.errors import BadFileKindError, ForbiddenFileError
     import bzrlib.branch
-    import bzrlib.osutils
 
     assert isinstance(recurse, bool)
 
@@ -129,10 +127,7 @@
         elif sub_tree:
             mutter("%r is a bzr tree" %f)
         else:
-            entry = inv.add_path(rf, kind=kind)
-            mutter("added %r kind %r file_id={%s}" % (rf, kind, entry.file_id))
-            count += 1 
-            reporter(rf, kind, entry)
+            count += __add_one(branch, inv, rf, kind, reporter)
 
         if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
@@ -152,3 +147,21 @@
         branch._write_inventory(inv)
 
     return count
+
+def __add_one(branch, inv, path, kind, reporter):
+    """Add a file or directory, automatically add unversioned parents."""
+
+    # Nothing to do if path is already versioned.
+    # This is safe from infinite recursion because the branch root is
+    # always versioned.
+    if inv.path2id(path) != None:
+        return 0
+
+    # add parent
+    count = __add_one(branch, inv, dirname(path), 'directory', reporter)
+
+    entry = inv.add_path(path, kind=kind)
+    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
+    reporter(path, kind, entry)
+
+    return count + 1

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-12 13:05:29 +0000
+++ bzrlib/branch.py	2005-09-12 13:48:32 +0000
@@ -28,9 +28,9 @@
 import bzrlib.errors
 from bzrlib.textui import show_status
 from bzrlib.revision import Revision
-from bzrlib.xml import unpack_xml
 from bzrlib.delta import compare_trees
 from bzrlib.tree import EmptyTree, RevisionTree
+import bzrlib.xml
 import bzrlib.ui
 
 
@@ -302,7 +302,6 @@
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import pack_xml
         
         os.mkdir(self.controlfilename([]))
         self.controlfile('README', 'w').write(
@@ -321,7 +320,9 @@
         # if we want per-tree root ids then this is the place to set
         # them; they're not needed for now and so ommitted for
         # simplicity.
-        pack_xml(Inventory(), self.controlfile('inventory','w'))
+        f = self.controlfile('inventory','w')
+        bzrlib.xml.serializer_v4.write_inventory(Inventory(), f)
+
 
     def _check_format(self):
         """Check this branch format is supported.
@@ -335,7 +336,7 @@
         # on Windows from Linux and so on.  I think it might be better
         # to always make all internal files in unix format.
         fmt = self.controlfile('branch-format', 'r').read()
-        fmt.replace('\r\n', '')
+        fmt = fmt.replace('\r\n', '\n')
         if fmt != BZR_BRANCH_FORMAT:
             raise BzrError('sorry, branch format %r not supported' % fmt,
                            ['use a different bzr version',
@@ -361,18 +362,12 @@
     def read_working_inventory(self):
         """Read the working inventory."""
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-        from time import time
-        before = time()
         self.lock_read()
         try:
             # ElementTree does its own conversion from UTF-8, so open in
             # binary.
-            inv = unpack_xml(Inventory,
-                             self.controlfile('inventory', 'rb'))
-            mutter("loaded inventory of %d items in %f"
-                   % (len(inv), time() - before))
-            return inv
+            f = self.controlfile('inventory', 'rb')
+            return bzrlib.xml.serializer_v4.read_inventory(f)
         finally:
             self.unlock()
             
@@ -384,13 +379,12 @@
         will be committed to the next revision.
         """
         from bzrlib.atomicfile import AtomicFile
-        from bzrlib.xml import pack_xml
         
         self.lock_write()
         try:
             f = AtomicFile(self.controlfilename('inventory'), 'wb')
             try:
-                pack_xml(inv, f)
+                bzrlib.xml.serializer_v4.write_inventory(inv, f)
                 f.commit()
             finally:
                 f.close()
@@ -581,7 +575,7 @@
             f.close()
 
 
-    def get_revision_xml(self, revision_id):
+    def get_revision_xml_file(self, revision_id):
         """Return XML file object for revision object."""
         if not revision_id or not isinstance(revision_id, basestring):
             raise InvalidRevisionId(revision_id)
@@ -596,12 +590,16 @@
             self.unlock()
 
 
+    #deprecated
+    get_revision_xml = get_revision_xml_file
+
+
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
-        xml_file = self.get_revision_xml(revision_id)
+        xml_file = self.get_revision_xml_file(revision_id)
 
         try:
-            r = unpack_xml(Revision, xml_file)
+            r = bzrlib.xml.serializer_v4.read_revision(xml_file)
         except SyntaxError, e:
             raise bzrlib.errors.BzrError('failed to unpack revision_xml',
                                          [revision_id,
@@ -652,14 +650,16 @@
                parameter which can be either an integer revno or a
                string hash."""
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
 
-        return unpack_xml(Inventory, self.get_inventory_xml(inventory_id))
+        f = self.get_inventory_xml_file(inventory_id)
+        return bzrlib.xml.serializer_v4.read_inventory(f)
 
 
     def get_inventory_xml(self, inventory_id):
         """Get inventory XML as a file object."""
         return self.inventory_store[inventory_id]
+
+    get_inventory_xml_file = get_inventory_xml
             
 
     def get_inventory_sha1(self, inventory_id):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-04 02:59:56 +0000
+++ bzrlib/builtins.py	2005-09-12 01:29:49 +0000
@@ -62,11 +62,10 @@
     directory is shown.  Otherwise, only the status of the specified
     files or directories is reported.  If a directory is given, status
     is reported for everything inside that directory.
+    """
 
-    If a revision is specified, the changes since that revision are shown.
-    """
     takes_args = ['file*']
-    takes_options = ['all', 'show-ids', 'revision']
+    takes_options = ['all', 'show-ids']
     aliases = ['st', 'stat']
     
     def run(self, all=False, show_ids=False, file_list=None):
@@ -92,8 +91,8 @@
     takes_args = ['revision_id']
     
     def run(self, revision_id):
-        from bzrlib.xml import pack_xml
-        pack_xml(find_branch('.').get_revision(revision_id), sys.stdout)
+        b = find_branch('.')
+        sys.stdout.write(b.get_revision_xml_file(revision_id).read())
 
 
 class cmd_revno(Command):
@@ -103,6 +102,7 @@
     def run(self):
         print find_branch('.').revno()
 
+
 class cmd_revision_info(Command):
     """Show revision number and revision id for a given revision identifier.
     """
@@ -144,8 +144,10 @@
     Therefore simply saying 'bzr add' will version all files that
     are currently unknown.
 
-    TODO: Perhaps adding a file whose directly is not versioned should
-    recursively add that parent, rather than giving an error?
+    Adding a file whose parent directory is not versioned will
+    implicitly add the parent, and so on up to the root. This means
+    you should never need to explictly add a directory, they'll just
+    get added when you add a file in the directory.
     """
     takes_args = ['file*']
     takes_options = ['verbose', 'no-recurse']
@@ -535,7 +537,7 @@
     examples:
         bzr diff
         bzr diff -r1
-        bzr diff -r1:2
+        bzr diff -r1..2
     """
     
     takes_args = ['file*']
@@ -1196,7 +1198,7 @@
         try:
             merge(other, base, check_clean=(not force), merge_type=merge_type)
         except bzrlib.errors.AmbiguousBase, e:
-            m = ("sorry, bzr can't determine the write merge base yet\n"
+            m = ("sorry, bzr can't determine the right merge base yet\n"
                  "candidates are:\n  "
                  + "\n  ".join(e.bases)
                  + "\n"

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-08-25 12:46:42 +0000
+++ bzrlib/commit.py	2005-09-05 09:27:11 +0000
@@ -16,8 +16,6 @@
 
 
 
-# FIXME: "bzr commit doc/format" commits doc/format.txt!
-
 def commit(branch, message,
            timestamp=None,
            timezone=None,
@@ -66,7 +64,7 @@
     from bzrlib.errors import BzrError, PointlessCommit
     from bzrlib.revision import Revision, RevisionReference
     from bzrlib.trace import mutter, note
-    from bzrlib.xml import pack_xml
+    from bzrlib.xml import serializer_v4
 
     branch.lock_write()
 
@@ -123,7 +121,8 @@
         inv_id = rev_id
 
         inv_tmp = tempfile.TemporaryFile()
-        pack_xml(new_inv, inv_tmp)
+        
+        serializer_v4.write_inventory(new_inv, inv_tmp)
         inv_tmp.seek(0)
         branch.inventory_store.add(inv_tmp, inv_id)
         mutter('new inventory_id is {%s}' % inv_id)
@@ -162,7 +161,7 @@
             rev.parents.append(RevisionReference(merge_rev))            
 
         rev_tmp = tempfile.TemporaryFile()
-        pack_xml(rev, rev_tmp)
+        serializer_v4.write_revision(rev, rev_tmp)
         rev_tmp.seek(0)
         branch.revision_store.add(rev_tmp, rev_id)
         mutter("new revision_id is {%s}" % rev_id)

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-09-01 09:17:38 +0000
+++ bzrlib/help.py	2005-09-05 04:08:03 +0000
@@ -70,7 +70,7 @@
 
     Only describes arguments, not options.
     """
-    s = cmd_object.name() + ' '
+    s = 'bzr ' + cmd_object.name() + ' '
     for aname in cmd_object.takes_args:
         aname = aname.upper()
         if aname[-1] in ['$', '+']:

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-08-25 12:46:42 +0000
+++ bzrlib/inventory.py	2005-09-06 07:26:13 +0000
@@ -100,7 +100,9 @@
     # directories, etc etc.
 
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', ]
+                 'text_id', 'parent_id', 'children',
+                 'text_version', 'entry_version', ]
+
 
     def __init__(self, file_id, name, kind, parent_id, text_id=None):
         """Create an InventoryEntry
@@ -117,12 +119,14 @@
         Traceback (most recent call last):
         BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
         """
+        assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
             raise BzrCheckError('InventoryEntry name %r is invalid' % name)
         
+        self.text_version = None
+        self.entry_version = None
         self.text_sha1 = None
         self.text_size = None
-    
         self.file_id = file_id
         self.name = name
         self.kind = kind
@@ -162,60 +166,6 @@
                    self.parent_id))
 
     
-    def to_element(self):
-        """Convert to XML element"""
-        from bzrlib.xml import Element
-        
-        e = Element('entry')
-
-        e.set('name', self.name)
-        e.set('file_id', self.file_id)
-        e.set('kind', self.kind)
-
-        if self.text_size != None:
-            e.set('text_size', '%d' % self.text_size)
-            
-        for f in ['text_id', 'text_sha1']:
-            v = getattr(self, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if self.parent_id != ROOT_ID:
-            assert isinstance(self.parent_id, basestring)
-            e.set('parent_id', self.parent_id)
-
-        e.tail = '\n'
-            
-        return e
-
-
-    def from_element(cls, elt):
-        assert elt.tag == 'entry'
-
-        ## original format inventories don't have a parent_id for
-        ## nodes in the root directory, but it's cleaner to use one
-        ## internally.
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-        self.text_id = elt.get('text_id')
-        self.text_sha1 = elt.get('text_sha1')
-        
-        ## mutter("read inventoryentry: %r" % (elt.attrib))
-
-        v = elt.get('text_size')
-        self.text_size = v and int(v)
-
-        return self
-            
-
-    from_element = classmethod(from_element)
-
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
             return NotImplemented
@@ -226,7 +176,9 @@
                and (self.text_size == other.text_size) \
                and (self.text_id == other.text_id) \
                and (self.parent_id == other.parent_id) \
-               and (self.kind == other.kind)
+               and (self.kind == other.kind) \
+               and (self.text_version == other.text_version) \
+               and (self.entry_version == other.entry_version)
 
 
     def __ne__(self, other):
@@ -416,7 +368,10 @@
         """Add entry to inventory.
 
         To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this."""
+        which calls this.
+
+        Returns the new entry object.
+        """
         if entry.file_id in self._byid:
             raise BzrError("inventory already contains entry with id {%s}" % entry.file_id)
 
@@ -440,7 +395,9 @@
     def add_path(self, relpath, kind, file_id=None):
         """Add entry from a path.
 
-        The immediate parent must already be versioned"""
+        The immediate parent must already be versioned.
+
+        Returns the new entry object."""
         from bzrlib.branch import gen_file_id
         
         parts = bzrlib.osutils.splitpath(relpath)
@@ -487,44 +444,6 @@
         del self[ie.parent_id].children[ie.name]
 
 
-    def to_element(self):
-        """Convert to XML Element"""
-        from bzrlib.xml import Element
-        
-        e = Element('inventory')
-        e.text = '\n'
-        if self.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', self.root.file_id)
-        for path, ie in self.iter_entries():
-            e.append(ie.to_element())
-        return e
-    
-
-    def from_element(cls, elt):
-        """Construct from XML Element
-        
-        >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('foo.c-123981239', 'foo.c', 'file', ROOT_ID))
-        InventoryEntry('foo.c-123981239', 'foo.c', kind='file', parent_id='TREE_ROOT')
-        >>> elt = inv.to_element()
-        >>> inv2 = Inventory.from_element(elt)
-        >>> inv2 == inv
-        True
-        """
-        # XXXX: doctest doesn't run this properly under python2.3
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        o = cls(root_id)
-        for e in elt:
-            ie = InventoryEntry.from_element(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            o.add(ie)
-        return o
-        
-    from_element = classmethod(from_element)
-
-
     def __eq__(self, other):
         """Compare two sets by comparing their contents.
 
@@ -559,7 +478,6 @@
         raise ValueError('not hashable')
 
 
-
     def get_idpath(self, file_id):
         """Return a list of file_ids for the path to an entry.
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-09-05 02:01:31 +0000
+++ bzrlib/merge.py	2005-09-07 01:34:07 +0000
@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-import os.path
+import os
 import tempfile
 import shutil
 import errno
@@ -357,7 +357,7 @@
             if path == '.':
                 path = ''
             else:
-                assert path.startswith('./'), "path is %s" % path
+                assert path.startswith('.' + os.sep), "path is %s" % path
             path = path[2:]
         adjust_ids.append((path, id))
     if len(adjust_ids) > 0:

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2005-09-01 11:27:20 +0000
+++ bzrlib/msgeditor.py	2005-09-07 01:34:07 +0000
@@ -20,6 +20,7 @@
 """Commit message editor support."""
 
 import os
+from subprocess import call
 from bzrlib.errors import BzrError
 
 def _get_editor():
@@ -30,12 +31,12 @@
     if e is not None:
         yield e
         
-    if os.name == "windows":
-        yield "notepad.exe"
-    elif os.name == "posix":
-        try:
-            yield os.environ["EDITOR"]
-        except KeyError:
+    try:
+        yield os.environ["EDITOR"]
+    except KeyError:
+        if os.name == "nt":
+            yield "notepad.exe"
+        elif os.name == "posix":
             yield "/usr/bin/vi"
 
 
@@ -43,8 +44,7 @@
     """Try to execute an editor to edit the commit message."""
     for e in _get_editor():
         edargs = e.split(' ')
-        x = os.spawnvp(os.P_WAIT, edargs[0],
-                       edargs + [filename])
+        x = call(edargs + [filename])
         if x == 0:
             return True
         elif x == 127:

=== modified file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-09-04 03:32:17 +0000
+++ bzrlib/remotebranch.py	2005-09-07 08:06:16 +0000
@@ -26,14 +26,14 @@
 
 import gzip
 from cStringIO import StringIO
+import os
 import urllib2
-
-from errors import BzrError, BzrCheckError
-from branch import Branch, BZR_BRANCH_FORMAT
-from trace import mutter
-
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
+import urlparse
+
+from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
+from bzrlib.trace import mutter
+from bzrlib.xml import serializer_v4
 
 
 ENABLE_URLGRABBER = False
@@ -84,8 +84,8 @@
     orig_url = url
     while True:
         try:
-            ff = get_url(url + '/.bzr/branch-format')
-
+            fmt_url = url + '/.bzr/branch-format'
+            ff = get_url(fmt_url)
             fmt = ff.read()
             ff.close()
 
@@ -98,12 +98,17 @@
         except urllib2.URLError:
             pass
 
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
+        scheme, host, path = list(urlparse.urlparse(url))[:3]
+        # discard params, query, fragment
+        
+        # strip off one component of the path component
+        idx = path.rfind('/')
+        if idx == -1 or path == '/':
+            raise BzrError('no branch root found for URL %s'
+                           ' or enclosing directories'
+                           % orig_url)
+        path = path[:idx]
+        url = urlparse.urlunparse((scheme, host, path, '', '', ''))
         
 
 
@@ -154,13 +159,11 @@
 
 
     def get_revision(self, revision_id):
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
         try:
             revf = self.revision_store[revision_id]
         except KeyError:
             raise NoSuchRevision(self, revision_id)
-        r = unpack_xml(Revision, revf)
+        r = serializer_v4.read_revision(revf)
         if r.revision_id != revision_id:
             raise BzrCheckError('revision stored as {%s} actually contains {%s}'
                                 % (revision_id, r.revision_id))

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-09-12 02:55:07 +0000
+++ bzrlib/revision.py	2005-09-12 08:21:24 +0000
@@ -70,90 +70,21 @@
     def __repr__(self):
         return "<Revision id %s>" % self.revision_id
 
-        
-    def to_element(self):
-        from bzrlib.xml import Element, SubElement
-        
-        root = Element('revision',
-                       committer = self.committer,
-                       timestamp = '%.9f' % self.timestamp,
-                       revision_id = self.revision_id,
-                       inventory_id = self.inventory_id,
-                       inventory_sha1 = self.inventory_sha1,
-                       )
-        if self.timezone:
-            root.set('timezone', str(self.timezone))
-        root.text = '\n'
-        
-        msg = SubElement(root, 'message')
-        msg.text = self.message
-        msg.tail = '\n'
-
-        if self.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in self.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-                if rr.revision_sha1:
-                    p.set('revision_sha1', rr.revision_sha1)
-
-        return root
-
-
-    def from_element(cls, elt):
-        return unpack_revision(elt)
-
-    from_element = classmethod(from_element)
-
-
-
-def unpack_revision(elt):
-    """Convert XML element into Revision object."""
-    # <changeset> is deprecated...
-    if elt.tag not in ('revision', 'changeset'):
-        raise bzrlib.errors.BzrError("unexpected tag in revision file: %r" % elt)
-
-    rev = Revision(committer = elt.get('committer'),
-                   timestamp = float(elt.get('timestamp')),
-                   revision_id = elt.get('revision_id'),
-                   inventory_id = elt.get('inventory_id'),
-                   inventory_sha1 = elt.get('inventory_sha1')
-                   )
-
-    precursor = elt.get('precursor')
-    precursor_sha1 = elt.get('precursor_sha1')
-
-    pelts = elt.find('parents')
-
-    if pelts:
-        for p in pelts:
-            assert p.tag == 'revision_ref', \
-                   "bad parent node tag %r" % p.tag
-            rev_ref = RevisionReference(p.get('revision_id'),
-                                        p.get('revision_sha1'))
-            rev.parents.append(rev_ref)
-
-        if precursor:
-            # must be consistent
-            prec_parent = rev.parents[0].revision_id
-            assert prec_parent == precursor
-    elif precursor:
-        # revisions written prior to 0.0.5 have a single precursor
-        # give as an attribute
-        rev_ref = RevisionReference(precursor, precursor_sha1)
-        rev.parents.append(rev_ref)
-
-    v = elt.get('timezone')
-    rev.timezone = v and int(v)
-
-    rev.message = elt.findtext('message') # text of <message>
-    return rev
-
-
+    def __eq__(self, other):
+        if not isinstance(other, Revision):
+            return False
+        return (self.inventory_id == other.inventory_id
+                and self.inventory_sha1 == other.inventory_sha1
+                and self.revision_id == other.revision_id
+                and self.timestamp == other.timestamp
+                and self.message == other.message
+                and self.timezone == other.timezone
+                and self.committer == other.committer)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+        
 
 REVISION_ID_RE = None
 

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-11 03:39:37 +0000
+++ bzrlib/selftest/__init__.py	2005-09-12 08:21:24 +0000
@@ -319,7 +319,6 @@
                    'bzrlib.selftest.testfetch',
                    'bzrlib.selftest.versioning',
                    'bzrlib.selftest.whitebox',
-                   'bzrlib.selftest.testmerge',
                    'bzrlib.selftest.testmerge3',
                    'bzrlib.selftest.testhashcache',
                    'bzrlib.selftest.teststatus',
@@ -331,6 +330,7 @@
                    'bzrlib.selftest.test_merge_core',
                    'bzrlib.selftest.test_smart_add',
                    'bzrlib.selftest.testdiff',
+                   'bzrlib.selftest.test_xml',
                    'bzrlib.fetch',
                    'bzrlib.selftest.teststore',
                    'bzrlib.selftest.testgraph',

=== added file 'bzrlib/selftest/test_xml.py'
--- bzrlib/selftest/test_xml.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_xml.py	2005-09-06 07:26:13 +0000
@@ -0,0 +1,140 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+
+from bzrlib.selftest import TestCase
+from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.xml import serializer_v4, serializer_v5
+
+_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
+<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
+<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
+<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
+</inventory>"""
+
+
+_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
+</parents>
+</revision>
+"""
+
+_revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92"/>
+</parents>
+</revision>
+"""
+
+_committed_inv_v5 = """<inventory>
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" 
+      name="bar" parent_id="TREE_ROOT" 
+      text_version="mbp@foo-123123" entry_version="mbp@foo-123123"
+      />
+<directory name="subdir"
+           file_id="foo-20050801201819-4139aa4a272f4250"
+           parent_id="TREE_ROOT" 
+           entry_version="mbp@foo-00"/>
+<file file_id="bar-20050824000535-6bc48cfad47ed134" 
+      name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
+      entry_version="mbp@foo-00"
+      text_version="mbp@foo-123123"/>
+</inventory>
+"""
+
+class TestSerializer(TestCase):
+    """Test XML serialization"""
+    def test_canned_inventory(self):
+        """Test unpacked a canned inventory v4 file."""
+        inp = StringIO(_working_inventory_v4)
+        inv = serializer_v4.read_inventory(inp)
+        self.assertEqual(len(inv), 4)
+        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
+
+    def test_unpack_revision(self):
+        """Test unpacking a canned revision v4"""
+        inp = StringIO(_revision_v4)
+        rev = serializer_v4.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parents), 1)
+        eq(rev.parents[0].revision_id,
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_revision_5(self):
+        """Test unpacking a canned revision v5"""
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parents), 1)
+        eq(rev.timezone, 36000)
+        eq(rev.parents[0].revision_id,
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_inventory_5(self):
+        """Unpack canned new-style inventory"""
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        eq = self.assertEqual
+        eq(len(inv), 4)
+        ie = inv['bar-20050824000535-6bc48cfad47ed134']
+        eq(ie.kind, 'file')
+        eq(ie.text_version, 'mbp@foo-123123')
+        eq(ie.entry_version, 'mbp@foo-00')
+        eq(ie.name, 'bar')
+        eq(inv[ie.parent_id].kind, 'directory')
+
+    def test_repack_inventory_5(self):
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        outp = StringIO()
+        serializer_v5.write_inventory(inv, outp)
+        inv2 = serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        self.assertEqual(inv, inv2)
+
+    def test_repack_revision_5(self):
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        rev2 = serializer_v5.read_revision(StringIO(outp.getvalue()))
+        self.assertEqual(rev, rev2)
+

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-08-25 12:46:42 +0000
+++ bzrlib/selftest/testinv.py	2005-09-05 08:00:35 +0000
@@ -49,3 +49,11 @@
         self.assertEqual(inv.path2id('src/bye.c'), 'bye-id')
         
         self.assert_('src-id' in inv)
+
+
+    def test_version(self):
+        """Inventory remembers the text's version."""
+        inv = Inventory()
+        ie = inv.add_path('foo.txt', 'file')
+        ## XXX
+

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-08-30 03:10:32 +0000
+++ bzrlib/selftest/versioning.py	2005-09-12 01:29:49 +0000
@@ -17,7 +17,6 @@
 
 """Tests of simple versioning operations"""
 
-# TODO: test adding a file whose directory is not versioned
 # TODO: test trying to commit within a directory that is not yet added
 
 
@@ -55,11 +54,11 @@
         self.assertEquals(delta.added[0][0], 'foo')
         self.failIf(delta.modified)
 
-    def test_add_in_unversioned(self):
+    def test_branch_add_in_unversioned(self):
         """Try to add a file in an unversioned directory.
 
-        smart_add may eventually add the parent as necessary, but simple
-        branch add doesn't do that.
+        "bzr add" adds the parent as necessary, but simple branch add
+        doesn't do that.
         """
         from bzrlib.branch import Branch
         from bzrlib.errors import NotVersionedError
@@ -75,7 +74,53 @@
         
         self.check_and_upgrade()
 
-        
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" should add the parent(s) as necessary.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
+        # Multiple unversioned parents
+        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), ['veil'])
+        run_bzr(['add', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), [])
+
+        # Check whacky paths work
+        self.build_tree(['cicatriz/', 'cicatriz/esp'])
+        eq(list(b.unknowns()), ['cicatriz'])
+        run_bzr(['add', 'inertiatic/../cicatriz/esp'])
+        eq(list(b.unknowns()), [])
+
+    def test_add_in_versioned(self):
+        """Try to add a file in a versioned directory.
+
+        "bzr add" should do this happily.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', '--no-recurse', 'inertiatic'])
+        eq(list(b.unknowns()), ['inertiatic/esp'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
     def test_subdir_add(self):
         """Add in subdirectory should add only things from there down"""
         
@@ -131,7 +176,6 @@
         
 
         
-        
 class SubdirCommit(TestCaseInTempDir):
 
     def test_subdir_commit(self):

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-08-27 04:42:41 +0000
+++ bzrlib/status.py	2005-09-07 23:14:30 +0000
@@ -21,16 +21,22 @@
                 show_ids=False,
                 to_file=None,
                 show_pending=True):
-    """Display single-line status for non-ignored working files.
+    """Display status for non-ignored working files.
 
-    show_all
-        If true, show unmodified files too.
+    show_unchanged
+        If set, includes unchanged files.
 
     specific_files
         If set, only show the status of files in this list.
 
+    show_ids
+        If set, includes each file's id.
+
     to_file
         If set, write to this file (default stdout.)
+
+    show_pending
+        If set, write pending merges.
     """
     import sys
     from bzrlib.delta import compare_trees

=== modified file 'bzrlib/store.py'
--- bzrlib/store.py	2005-09-04 02:59:56 +0000
+++ bzrlib/store.py	2005-09-06 23:19:08 +0000
@@ -68,10 +68,12 @@
     def __init__(self, basedir):
         self._basedir = basedir
 
-    def _path(self, id):
-        if '\\' in id or '/' in id:
-            raise ValueError("invalid store id %r" % id)
-        return os.path.join(self._basedir, id)
+    def _path(self, entry_id):
+        if not isinstance(entry_id, basestring):
+            raise TypeError(type(entry_id))
+        if '\\' in entry_id or '/' in entry_id:
+            raise ValueError("invalid store id %r" % entry_id)
+        return os.path.join(self._basedir, entry_id)
 
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self._basedir)

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-08-29 10:57:01 +0000
+++ bzrlib/upgrade.py	2005-09-05 09:27:11 +0000
@@ -15,8 +15,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import tempfile, os, errno
+                
 import bzrlib.errors
 import bzrlib.progress
+from bzrlib.xml import serializer_v4
 
 
 def upgrade(branch):
@@ -96,11 +99,8 @@
                 # revision entry. I'm not supremely happy about it, but
                 # there should be *some* way of making old entries have
                 # the full meta information.
-                import tempfile, os, errno
-                from bzrlib.xml import pack_xml
-                
                 rev_tmp = tempfile.TemporaryFile()
-                pack_xml(rev, rev_tmp)
+                serializer_v4.write_revision(rev, rev_tmp)
                 rev_tmp.seek(0)
 
                 tmpfd, tmp_path = tempfile.mkstemp(prefix=rev_id, suffix='.gz',

=== modified file 'bzrlib/xml.py'
--- bzrlib/xml.py	2005-08-25 05:58:05 +0000
+++ bzrlib/xml.py	2005-09-06 06:44:11 +0000
@@ -20,21 +20,344 @@
 # "XML is like violence: if it doesn't solve your problem, you aren't
 # using enough of it." -- various
 
-# importing this module is fairly slow because it has to load several ElementTree bits
+# importing this module is fairly slow because it has to load several
+# ElementTree bits
+
 try:
     from util.cElementTree import ElementTree, SubElement, Element
 except ImportError:
     from util.elementtree.ElementTree import ElementTree, SubElement, Element
 
-
-def pack_xml(o, f):
-    """Write object o to file f as XML.
-
-    o must provide a to_element method.
-    """
-    ElementTree(o.to_element()).write(f, 'utf-8')
-    f.write('\n')
-
-
-def unpack_xml(cls, f):
-    return cls.from_element(ElementTree().parse(f))
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+from bzrlib.revision import Revision, RevisionReference        
+from bzrlib.errors import BzrError
+
+
+class Serializer(object):
+    """Abstract object serialize/deserialize"""
+    def write_inventory(self, inv, f):
+        """Write inventory to a file"""
+        elt = self._pack_inventory(inv)
+        self._write_element(elt, f)
+
+    def read_inventory(self, f):
+        return self._unpack_inventory(self._read_element(f))
+
+    def write_revision(self, rev, f):
+        self._write_element(self._pack_revision(rev), f)
+
+    def read_revision(self, f):
+        return self._unpack_revision(self._read_element(f))
+
+    def _write_element(self, elt, f):
+        ElementTree(elt).write(f, 'utf-8')
+        f.write('\n')
+
+    def _read_element(self, f):
+        return ElementTree().parse(f)
+
+
+
+class _Serializer_v4(Serializer):
+    """Version 0.0.4 serializer
+
+    You should use the serialzer_v4 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        e = Element('entry')
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+        e.set('kind', ie.kind)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_id', 'text_sha1']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        assert elt.tag == 'entry'
+
+        ## original format inventories don't have a parent_id for
+        ## nodes in the root directory, but it's cleaner to use one
+        ## internally.
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        ie = InventoryEntry(elt.get('file_id'),
+                              elt.get('name'),
+                              elt.get('kind'),
+                              parent_id)
+        ie.text_id = elt.get('text_id')
+        ie.text_sha1 = elt.get('text_sha1')
+
+        ## mutter("read inventoryentry: %r" % (elt.attrib))
+
+        v = elt.get('text_size')
+        ie.text_size = v and int(v)
+
+        return ie
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for rr in rev.parents:
+                assert isinstance(rr, RevisionReference)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert rr.revision_id
+                p.set('revision_id', rr.revision_id)
+                if rr.revision_sha1:
+                    p.set('revision_sha1', rr.revision_sha1)
+
+        return root
+
+    
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        
+        # <changeset> is deprecated...
+        if elt.tag not in ('revision', 'changeset'):
+            raise BzrError("unexpected tag in revision file: %r" % elt)
+
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        precursor = elt.get('precursor')
+        precursor_sha1 = elt.get('precursor_sha1')
+
+        pelts = elt.find('parents')
+
+        if pelts:
+            for p in pelts:
+                assert p.tag == 'revision_ref', \
+                       "bad parent node tag %r" % p.tag
+                rev_ref = RevisionReference(p.get('revision_id'),
+                                            p.get('revision_sha1'))
+                rev.parents.append(rev_ref)
+
+            if precursor:
+                # must be consistent
+                prec_parent = rev.parents[0].revision_id
+                assert prec_parent == precursor
+        elif precursor:
+            # revisions written prior to 0.0.5 have a single precursor
+            # give as an attribute
+            rev_ref = RevisionReference(precursor, precursor_sha1)
+            rev.parents.append(rev_ref)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+class _Serializer_v5(Serializer):
+    """Version 5 serializer
+
+    Packs objects into XML and vice versa.
+
+    You should use the serialzer_v5 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        assert ie.kind == 'directory' or ie.kind == 'file'
+        e = Element(ie.kind)
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_version', 'text_sha1', 'entry_version']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for rr in rev.parents:
+                assert isinstance(rr, RevisionReference)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert rr.revision_id
+                p.set('revision_id', rr.revision_id)
+
+        return root
+
+    
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        kind = elt.tag
+        assert kind == 'directory' or kind == 'file'
+
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        ie = InventoryEntry(elt.get('file_id'),
+                            elt.get('name'),
+                            kind,
+                            parent_id)
+        ie.text_version = elt.get('text_version')
+        ie.entry_version = elt.get('entry_version')
+        ie.text_sha1 = elt.get('text_sha1')
+        v = elt.get('text_size')
+        ie.text_size = v and int(v)
+
+        return ie
+
+
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        assert elt.tag == 'revision'
+        
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        for p in elt.find('parents'):
+            assert p.tag == 'revision_ref', \
+                   "bad parent node tag %r" % p.tag
+            rev_ref = RevisionReference(p.get('revision_id'))
+            rev.parents.append(rev_ref)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+"""singleton instance"""
+serializer_v4 = _Serializer_v4()
+
+serializer_v5 = _Serializer_v5()

